@@ -507,7 +507,7 @@ backfitting <- function(z, base_learners, base_learners_list, data, wz, phi_gam_
       data_loop <- cbind(partial_resid = partial_resid, data[pred_vars])
       # (A) GeDS base-learners
       if (base_learners[[bl_name]]$type == "GeDS") {
-        max.intknots <- max.intknots <- if (length(pred_vars) == 1) {internal_knots + q
+        max.intknots <- max.intknots <- if (length(pred_vars) == 1) {internal_knots
         } else if (length(pred_vars) == 2 && internal_knots == 0) {stop("internal_knots must be > 0 for bivariate learners")
             } else {internal_knots}
         
