@@ -256,13 +256,13 @@ knots.GeDS <- function(Fn, n = 3L, options = c("all","internal"), ...)
   
   # 1. Linear
   if(n == 2L) {
-    kn <- Fn$Linear.Knots
+    kn <- Fn$Linear.IntKnots
   # 2. Quadratic
     } else if (n == 3L) {
-      kn <- Fn$Quadratic.Knots
+      kn <- Fn$Quadratic.IntKnots
   # 3. Cubic
       } else if (n == 4L) {
-        kn <- Fn$Cubic.Knots
+        kn <- Fn$Cubic.IntKnots
       }
   
   if (options == "all") {
@@ -380,7 +380,7 @@ predict.GeDS <- function(object, newdata,
     
     # Knots
     kn <- knots(object, n = n, options = "internal")
-    if (min(X) < object$Args$extr[1] || object$Args$extr[2] > max(kn)) warning("Input values out of the boundary knots")
+    if (min(X) < object$Args$extr[1] || object$Args$extr[2] < max(X)) warning("Input values out of the boundary knots")
     # Design matrix
     matrice <- splineDesign(knots = sort(c(kn, rep(range(X), n))), derivs = rep(0,length(X)), x = X, ord = n, outer.ok = T)
     
